import pytest
from fastapi import status
from app.core.exceptions import (
    ListNotFoundError,
    ListPermissionError,
    ListCreatorRequiredError,
    GroupNotFoundError,
    GroupPermissionError,
    GroupMembershipError,
    GroupOperationError,
    GroupValidationError,
    ItemNotFoundError,
    UserNotFoundError,
    InvalidOperationError,
    DatabaseConnectionError,
    DatabaseIntegrityError,
    DatabaseTransactionError,
    DatabaseQueryError,
    OCRServiceUnavailableError,
    OCRServiceConfigError,
    OCRUnexpectedError,
    OCRQuotaExceededError,
    InvalidFileTypeError,
    FileTooLargeError,
    OCRProcessingError,
    EmailAlreadyRegisteredError,
    UserCreationError,
    InviteNotFoundError,
    InviteExpiredError,
    InviteAlreadyUsedError,
    InviteCreationError,
    ListStatusNotFoundError,
    ConflictError,
    InvalidCredentialsError,
    NotAuthenticatedError,
    JWTError,
    JWTUnexpectedError
)
# TODO: It seems like settings are used in some exceptions.
# You will need to mock app.config.settings for these tests to pass.
# Consider using pytest-mock or unittest.mock.patch.
# Example: from app.config import settings


def test_list_not_found_error():
    list_id = 123
    with pytest.raises(ListNotFoundError) as excinfo:
        raise ListNotFoundError(list_id=list_id)
    assert excinfo.value.status_code == status.HTTP_404_NOT_FOUND
    assert excinfo.value.detail == f"List {list_id} not found"

def test_list_permission_error():
    list_id = 456
    action = "delete"
    with pytest.raises(ListPermissionError) as excinfo:
        raise ListPermissionError(list_id=list_id, action=action)
    assert excinfo.value.status_code == status.HTTP_403_FORBIDDEN
    assert excinfo.value.detail == f"You do not have permission to {action} list {list_id}"

def test_list_permission_error_default_action():
    list_id = 789
    with pytest.raises(ListPermissionError) as excinfo:
        raise ListPermissionError(list_id=list_id)
    assert excinfo.value.status_code == status.HTTP_403_FORBIDDEN
    assert excinfo.value.detail == f"You do not have permission to access list {list_id}"

def test_list_creator_required_error():
    list_id = 101
    action = "update"
    with pytest.raises(ListCreatorRequiredError) as excinfo:
        raise ListCreatorRequiredError(list_id=list_id, action=action)
    assert excinfo.value.status_code == status.HTTP_403_FORBIDDEN
    assert excinfo.value.detail == f"Only the list creator can {action} list {list_id}"

def test_group_not_found_error():
    group_id = 202
    with pytest.raises(GroupNotFoundError) as excinfo:
        raise GroupNotFoundError(group_id=group_id)
    assert excinfo.value.status_code == status.HTTP_404_NOT_FOUND
    assert excinfo.value.detail == f"Group {group_id} not found"

def test_group_permission_error():
    group_id = 303
    action = "invite"
    with pytest.raises(GroupPermissionError) as excinfo:
        raise GroupPermissionError(group_id=group_id, action=action)
    assert excinfo.value.status_code == status.HTTP_403_FORBIDDEN
    assert excinfo.value.detail == f"You do not have permission to {action} in group {group_id}"

def test_group_membership_error():
    group_id = 404
    action = "post"
    with pytest.raises(GroupMembershipError) as excinfo:
        raise GroupMembershipError(group_id=group_id, action=action)
    assert excinfo.value.status_code == status.HTTP_403_FORBIDDEN
    assert excinfo.value.detail == f"You must be a member of group {group_id} to {action}"

def test_group_membership_error_default_action():
    group_id = 505
    with pytest.raises(GroupMembershipError) as excinfo:
        raise GroupMembershipError(group_id=group_id)
    assert excinfo.value.status_code == status.HTTP_403_FORBIDDEN
    assert excinfo.value.detail == f"You must be a member of group {group_id} to access"

def test_group_operation_error():
    detail_msg = "Failed to perform group operation."
    with pytest.raises(GroupOperationError) as excinfo:
        raise GroupOperationError(detail=detail_msg)
    assert excinfo.value.status_code == status.HTTP_500_INTERNAL_SERVER_ERROR
    assert excinfo.value.detail == detail_msg

def test_group_validation_error():
    detail_msg = "Invalid group data."
    with pytest.raises(GroupValidationError) as excinfo:
        raise GroupValidationError(detail=detail_msg)
    assert excinfo.value.status_code == status.HTTP_400_BAD_REQUEST
    assert excinfo.value.detail == detail_msg

def test_item_not_found_error():
    item_id = 606
    with pytest.raises(ItemNotFoundError) as excinfo:
        raise ItemNotFoundError(item_id=item_id)
    assert excinfo.value.status_code == status.HTTP_404_NOT_FOUND
    assert excinfo.value.detail == f"Item {item_id} not found"

def test_user_not_found_error_no_identifier():
    with pytest.raises(UserNotFoundError) as excinfo:
        raise UserNotFoundError()
    assert excinfo.value.status_code == status.HTTP_404_NOT_FOUND
    assert excinfo.value.detail == "User not found."

def test_user_not_found_error_with_id():
    user_id = 707
    with pytest.raises(UserNotFoundError) as excinfo:
        raise UserNotFoundError(user_id=user_id)
    assert excinfo.value.status_code == status.HTTP_404_NOT_FOUND
    assert excinfo.value.detail == f"User with ID {user_id} not found."

def test_user_not_found_error_with_identifier_string():
    identifier = "test_user"
    with pytest.raises(UserNotFoundError) as excinfo:
        raise UserNotFoundError(identifier=identifier)
    assert excinfo.value.status_code == status.HTTP_404_NOT_FOUND
    assert excinfo.value.detail == f"User with identifier '{identifier}' not found."

def test_invalid_operation_error():
    detail_msg = "This operation is not allowed."
    with pytest.raises(InvalidOperationError) as excinfo:
        raise InvalidOperationError(detail=detail_msg)
    assert excinfo.value.status_code == status.HTTP_400_BAD_REQUEST
    assert excinfo.value.detail == detail_msg

def test_invalid_operation_error_custom_status():
    detail_msg = "This operation is forbidden."
    custom_status = status.HTTP_403_FORBIDDEN
    with pytest.raises(InvalidOperationError) as excinfo:
        raise InvalidOperationError(detail=detail_msg, status_code=custom_status)
    assert excinfo.value.status_code == custom_status
    assert excinfo.value.detail == detail_msg

# The following exceptions depend on `settings`
# We need to mock `app.config.settings` for these tests.
# For now, I will add placeholder tests that would fail without mocking.
# Consider using pytest-mock or unittest.mock.patch for this.

# def test_database_connection_error(mocker):
#     mocker.patch("app.core.exceptions.settings.DB_CONNECTION_ERROR", "Test DB connection error")
#     with pytest.raises(DatabaseConnectionError) as excinfo:
#         raise DatabaseConnectionError()
#     assert excinfo.value.status_code == status.HTTP_503_SERVICE_UNAVAILABLE
#     assert excinfo.value.detail == "Test DB connection error" # settings.DB_CONNECTION_ERROR

# def test_database_integrity_error(mocker):
#     mocker.patch("app.core.exceptions.settings.DB_INTEGRITY_ERROR", "Test DB integrity error")
#     with pytest.raises(DatabaseIntegrityError) as excinfo:
#         raise DatabaseIntegrityError()
#     assert excinfo.value.status_code == status.HTTP_400_BAD_REQUEST
#     assert excinfo.value.detail == "Test DB integrity error" # settings.DB_INTEGRITY_ERROR

# def test_database_transaction_error(mocker):
#     mocker.patch("app.core.exceptions.settings.DB_TRANSACTION_ERROR", "Test DB transaction error")
#     with pytest.raises(DatabaseTransactionError) as excinfo:
#         raise DatabaseTransactionError()
#     assert excinfo.value.status_code == status.HTTP_500_INTERNAL_SERVER_ERROR
#     assert excinfo.value.detail == "Test DB transaction error" # settings.DB_TRANSACTION_ERROR

# def test_database_query_error(mocker):
#     mocker.patch("app.core.exceptions.settings.DB_QUERY_ERROR", "Test DB query error")
#     with pytest.raises(DatabaseQueryError) as excinfo:
#         raise DatabaseQueryError()
#     assert excinfo.value.status_code == status.HTTP_500_INTERNAL_SERVER_ERROR
#     assert excinfo.value.detail == "Test DB query error" # settings.DB_QUERY_ERROR

# def test_ocr_service_unavailable_error(mocker):
#     mocker.patch("app.core.exceptions.settings.OCR_SERVICE_UNAVAILABLE", "Test OCR unavailable")
#     with pytest.raises(OCRServiceUnavailableError) as excinfo:
#         raise OCRServiceUnavailableError()
#     assert excinfo.value.status_code == status.HTTP_503_SERVICE_UNAVAILABLE
#     assert excinfo.value.detail == "Test OCR unavailable" # settings.OCR_SERVICE_UNAVAILABLE

# def test_ocr_service_config_error(mocker):
#     mocker.patch("app.core.exceptions.settings.OCR_SERVICE_CONFIG_ERROR", "Test OCR config error")
#     with pytest.raises(OCRServiceConfigError) as excinfo:
#         raise OCRServiceConfigError()
#     assert excinfo.value.status_code == status.HTTP_500_INTERNAL_SERVER_ERROR
#     assert excinfo.value.detail == "Test OCR config error" # settings.OCR_SERVICE_CONFIG_ERROR

# def test_ocr_unexpected_error(mocker):
#     mocker.patch("app.core.exceptions.settings.OCR_UNEXPECTED_ERROR", "Test OCR unexpected error")
#     with pytest.raises(OCRUnexpectedError) as excinfo:
#         raise OCRUnexpectedError()
#     assert excinfo.value.status_code == status.HTTP_500_INTERNAL_SERVER_ERROR
#     assert excinfo.value.detail == "Test OCR unexpected error" # settings.OCR_UNEXPECTED_ERROR

# def test_ocr_quota_exceeded_error(mocker):
#     mocker.patch("app.core.exceptions.settings.OCR_QUOTA_EXCEEDED", "Test OCR quota exceeded")
#     with pytest.raises(OCRQuotaExceededError) as excinfo:
#         raise OCRQuotaExceededError()
#     assert excinfo.value.status_code == status.HTTP_429_TOO_MANY_REQUESTS
#     assert excinfo.value.detail == "Test OCR quota exceeded" # settings.OCR_QUOTA_EXCEEDED

# def test_invalid_file_type_error(mocker):
#     test_types = ["png", "jpg"]
#     mocker.patch("app.core.exceptions.settings.ALLOWED_IMAGE_TYPES", test_types)
#     mocker.patch("app.core.exceptions.settings.OCR_INVALID_FILE_TYPE", "Invalid type: {types}")
#     with pytest.raises(InvalidFileTypeError) as excinfo:
#         raise InvalidFileTypeError()
#     assert excinfo.value.status_code == status.HTTP_400_BAD_REQUEST
#     assert excinfo.value.detail == f"Invalid type: {', '.join(test_types)}" # settings.OCR_INVALID_FILE_TYPE.format(types=", ".join(settings.ALLOWED_IMAGE_TYPES))

# def test_file_too_large_error(mocker):
#     max_size = 10
#     mocker.patch("app.core.exceptions.settings.MAX_FILE_SIZE_MB", max_size)
#     mocker.patch("app.core.exceptions.settings.OCR_FILE_TOO_LARGE", "File too large: {size}MB")
#     with pytest.raises(FileTooLargeError) as excinfo:
#         raise FileTooLargeError()
#     assert excinfo.value.status_code == status.HTTP_400_BAD_REQUEST
#     assert excinfo.value.detail == f"File too large: {max_size}MB" # settings.OCR_FILE_TOO_LARGE.format(size=settings.MAX_FILE_SIZE_MB)

# def test_ocr_processing_error(mocker):
#     error_detail = "Specific OCR error"
#     mocker.patch("app.core.exceptions.settings.OCR_PROCESSING_ERROR", "OCR processing failed: {detail}")
#     with pytest.raises(OCRProcessingError) as excinfo:
#         raise OCRProcessingError(detail=error_detail)
#     assert excinfo.value.status_code == status.HTTP_400_BAD_REQUEST
#     assert excinfo.value.detail == f"OCR processing failed: {error_detail}" # settings.OCR_PROCESSING_ERROR.format(detail=detail)


def test_email_already_registered_error():
    with pytest.raises(EmailAlreadyRegisteredError) as excinfo:
        raise EmailAlreadyRegisteredError()
    assert excinfo.value.status_code == status.HTTP_400_BAD_REQUEST
    assert excinfo.value.detail == "Email already registered."

def test_user_creation_error():
    with pytest.raises(UserCreationError) as excinfo:
        raise UserCreationError()
    assert excinfo.value.status_code == status.HTTP_500_INTERNAL_SERVER_ERROR
    assert excinfo.value.detail == "An error occurred during user creation."

def test_invite_not_found_error():
    invite_code = "TESTCODE123"
    with pytest.raises(InviteNotFoundError) as excinfo:
        raise InviteNotFoundError(invite_code=invite_code)
    assert excinfo.value.status_code == status.HTTP_404_NOT_FOUND
    assert excinfo.value.detail == f"Invite code {invite_code} not found"

def test_invite_expired_error():
    invite_code = "EXPIREDCODE"
    with pytest.raises(InviteExpiredError) as excinfo:
        raise InviteExpiredError(invite_code=invite_code)
    assert excinfo.value.status_code == status.HTTP_410_GONE
    assert excinfo.value.detail == f"Invite code {invite_code} has expired"

def test_invite_already_used_error():
    invite_code = "USEDCODE"
    with pytest.raises(InviteAlreadyUsedError) as excinfo:
        raise InviteAlreadyUsedError(invite_code=invite_code)
    assert excinfo.value.status_code == status.HTTP_410_GONE
    assert excinfo.value.detail == f"Invite code {invite_code} has already been used"

def test_invite_creation_error():
    group_id = 909
    with pytest.raises(InviteCreationError) as excinfo:
        raise InviteCreationError(group_id=group_id)
    assert excinfo.value.status_code == status.HTTP_500_INTERNAL_SERVER_ERROR
    assert excinfo.value.detail == f"Failed to create invite for group {group_id}"

def test_list_status_not_found_error():
    list_id = 808
    with pytest.raises(ListStatusNotFoundError) as excinfo:
        raise ListStatusNotFoundError(list_id=list_id)
    assert excinfo.value.status_code == status.HTTP_404_NOT_FOUND
    assert excinfo.value.detail == f"Status for list {list_id} not found"

def test_conflict_error():
    detail_msg = "Resource version mismatch."
    with pytest.raises(ConflictError) as excinfo:
        raise ConflictError(detail=detail_msg)
    assert excinfo.value.status_code == status.HTTP_409_CONFLICT
    assert excinfo.value.detail == detail_msg

# Tests for auth-related exceptions that likely require mocking app.config.settings

# def test_invalid_credentials_error(mocker):
#     mocker.patch("app.core.exceptions.settings.AUTH_INVALID_CREDENTIALS", "Invalid test credentials")
#     mocker.patch("app.core.exceptions.settings.AUTH_HEADER_NAME", "X-Test-Auth")
#     mocker.patch("app.core.exceptions.settings.AUTH_HEADER_PREFIX", "TestBearer")
#     with pytest.raises(InvalidCredentialsError) as excinfo:
#         raise InvalidCredentialsError()
#     assert excinfo.value.status_code == status.HTTP_401_UNAUTHORIZED
#     assert excinfo.value.detail == "Invalid test credentials"
#     assert excinfo.value.headers == {"X-Test-Auth": "TestBearer error=\"invalid_credentials\""}

# def test_not_authenticated_error(mocker):
#     mocker.patch("app.core.exceptions.settings.AUTH_NOT_AUTHENTICATED", "Not authenticated test")
#     mocker.patch("app.core.exceptions.settings.AUTH_HEADER_NAME", "X-Test-Auth")
#     mocker.patch("app.core.exceptions.settings.AUTH_HEADER_PREFIX", "TestBearer")
#     with pytest.raises(NotAuthenticatedError) as excinfo:
#         raise NotAuthenticatedError()
#     assert excinfo.value.status_code == status.HTTP_401_UNAUTHORIZED
#     assert excinfo.value.detail == "Not authenticated test"
#     assert excinfo.value.headers == {"X-Test-Auth": "TestBearer error=\"not_authenticated\""}

# def test_jwt_error(mocker):
#     error_msg = "Test JWT issue"
#     mocker.patch("app.core.exceptions.settings.JWT_ERROR", "JWT error: {error}")
#     mocker.patch("app.core.exceptions.settings.AUTH_HEADER_NAME", "X-Test-Auth")
#     mocker.patch("app.core.exceptions.settings.AUTH_HEADER_PREFIX", "TestBearer")
#     with pytest.raises(JWTError) as excinfo:
#         raise JWTError(error=error_msg)
#     assert excinfo.value.status_code == status.HTTP_401_UNAUTHORIZED
#     assert excinfo.value.detail == f"JWT error: {error_msg}"
#     assert excinfo.value.headers == {"X-Test-Auth": "TestBearer error=\"invalid_token\""}

# def test_jwt_unexpected_error(mocker):
#     error_msg = "Unexpected test JWT issue"
#     mocker.patch("app.core.exceptions.settings.JWT_UNEXPECTED_ERROR", "Unexpected JWT error: {error}")
#     mocker.patch("app.core.exceptions.settings.AUTH_HEADER_NAME", "X-Test-Auth")
#     mocker.patch("app.core.exceptions.settings.AUTH_HEADER_PREFIX", "TestBearer")
#     with pytest.raises(JWTUnexpectedError) as excinfo:
#         raise JWTUnexpectedError(error=error_msg)
#     assert excinfo.value.status_code == status.HTTP_401_UNAUTHORIZED
#     assert excinfo.value.detail == f"Unexpected JWT error: {error_msg}"
#     assert excinfo.value.headers == {"X-Test-Auth": "TestBearer error=\"invalid_token\""}
