import { boot } from 'quasar/wrappers';
import axios, { type AxiosInstance } from 'axios';
import { useAuthStore } from 'stores/auth';

declare module '@vue/runtime-core' {
  interface ComponentCustomProperties {
    $axios: AxiosInstance;
  }
}

const api = axios.create({
  baseURL: `${import.meta.env.VITE_API_URL || 'http://localhost:8000'}/api/v1`,
  headers: {
    'Content-Type': 'application/json',
  },
});

// Request interceptor for adding auth token
api.interceptors.request.use(
  (config) => {
    const authStore = useAuthStore();
    if (authStore.accessToken) {
      config.headers.Authorization = `Bearer ${authStore.accessToken}`;
    }
    return config;
  },
  (error) => {
    return Promise.reject(new Error(error.message));
  },
);

// Response interceptor for handling errors
api.interceptors.response.use(
  (response) => response,
  async (error) => {
    const authStore = useAuthStore();

    // If the error is 401 and we have a refresh token, try to refresh the access token
    if (error.response?.status === 401 && authStore.refreshToken) {
      try {
        await authStore.refreshAccessToken();
        // Retry the original request
        const config = error.config;
        config.headers.Authorization = `Bearer ${authStore.accessToken}`;
        return api(config);
      } catch (error) {
        // If refresh fails, clear tokens and redirect to login
        authStore.logout();
        window.location.href = '/login';
        return Promise.reject(
          new Error(error instanceof Error ? error.message : 'Failed to refresh token'),
        );
      }
    }

    // If it's a 401 without refresh token or refresh failed, clear tokens and redirect
    if (error.response?.status === 401) {
      authStore.logout();
      window.location.href = '/login';
    }

    return Promise.reject(new Error(error.response?.data?.detail || error.message));
  },
);

export default boot(({ app }) => {
  app.config.globalProperties.$axios = api;
});

export { api };
